# -*- coding: utf-8 -*-


def substr_unfiltered(suf_tree):
    # Parsing the tree
    # Recursively build suffix tree dictionary
    
    # Turn dictionary into list
    pass

def substr_unfiltered_recurse():
    
    # Using memoization to recursively build dictionary of substrings
    
    pass